/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import picto.app.interfaces.Status;
import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.SwingUtilities;
import picto.app.gui.PlayingBoard;
import picto.app.gui.PreviewWindow;
import picto.app.interfaces.Updatable;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class ColoredButton extends JButton implements Updatable, MouseListener{
    private static final long serialVersionUID = 56456565765324256L;
    private static Color colorButtonBorderColor = Config.getColor("colorButtonBorderColor");
    private static Color colorButtonBackgroundEmpty = Config.getColor("colorButtonBackgroundEmpty");
    private static Color colorButtonBackgroundNothing = Config.getColor("colorButtonBackgroundNothing");
    private static Color colorButtonBackgroundSomething = Config.getColor("colorButtonBackgroundSomething");
    private static boolean drag_enabled = Config.isDraggingEnabled();
    private static byte fist_click;
    private static boolean left_pressed = false;
    private static boolean right_pressed = false;
    private static boolean gameOver = false;
    private static PlayingBoard container = null;
    private byte curr_stat = Status.UNKNOWN;
    private final byte ii;
    private final byte jj;

    public ColoredButton(byte i, byte j){
        super();
        this.ii = i;
        this.jj = j;
        setBorder(i, j);
        setFocusable(false);
        setDoubleBuffered(false);
        setContentAreaFilled(false);
        setBackground(colorButtonBackgroundEmpty);
        setOpaque(true);
        addMouseListener(this);
    }

    /**
     * Sets the border of the cell according to it's position, this way it knows
     * if the border must be double or single.
     *
     * @param i
     * @param j
     */
    private void setBorder(int i, int j){
        setBorder(BorderFactory.createMatteBorder(
                1, 1, i % 5 == 4 ? 1 : 0, j % 5 == 4 ? 1 : 0,  
                colorButtonBorderColor));
    }
    
    /**
     * Sets the background of the cell
     *
     * @param col color to be used
     */
    public void setColor(Color col){
        setBackground(col);
    }

    /**
     * Paints the background according to the current status
     */
    @Override 
    public void update(){
        colorButtonBackgroundEmpty = Config.getColor("colorButtonBackgroundEmpty");
        colorButtonBackgroundNothing = Config.getColor("colorButtonBackgroundNothing");
        colorButtonBackgroundSomething = Config.getColor("colorButtonBackgroundSomething");
        
        switch (curr_stat){
            case FULL:
                setBackground(colorButtonBackgroundSomething);
                break;
            case EMPTY:
                setBackground(colorButtonBackgroundNothing);
                break;
            case UNKNOWN:
                setBackground(colorButtonBackgroundEmpty);
                break;
            default:
                assert false : "Unknown type of cell";
        }
        
        PreviewWindow.getInstance().pp.setElement(ii, jj, curr_stat == FULL);
        setBorder(ii, jj);
    }

    public static void setContainer(PlayingBoard board){
        ColoredButton.container = board;
    }

    @Override 
    public byte getStatus(){
        return curr_stat;
    }

    @Override 
    public void setStatus(byte stat){
        curr_stat = stat;
        update();
    }

    /**
     * What to do if you do a left click
     */
    private void leftClickPressed(){
        if (curr_stat != fist_click){
            return;
        }
        if (curr_stat != FULL){
            curr_stat = FULL;
            PreviewWindow.getInstance().pp.setElement(ii, jj, curr_stat == FULL);
            setBackground(colorButtonBackgroundSomething);
        } else {
            curr_stat = UNKNOWN;
            PreviewWindow.getInstance().pp.setElement(ii, jj, curr_stat == FULL);
            setBackground(colorButtonBackgroundEmpty);
        }
    }

    /**
     * What to do if you do a right click
     */
    private void rightClickPressed(){
        if (curr_stat != fist_click){
            return;
        }
        if (curr_stat != EMPTY){
            curr_stat = EMPTY;
            PreviewWindow.getInstance().pp.setElement(ii, jj, curr_stat == FULL);
            setBackground(colorButtonBackgroundNothing);
        } else {
            curr_stat = UNKNOWN;
            PreviewWindow.getInstance().pp.setElement(ii, jj, curr_stat == FULL);
            setBackground(colorButtonBackgroundEmpty);
        }
    }

    @Override 
    public void reset(){
        setStatus(UNKNOWN);
    }

    /**
     * Tells if mouse dragging is enabled
     *
     * @return <code>true</code> dragging is enabled and <code>false</code>
     * if it isn't
     */
    public static boolean isDragEnabled() {
        return drag_enabled;
    }

    public static void setDragEnabled(boolean drag_enabled) {
        ColoredButton.drag_enabled = drag_enabled;
    }

    public static void setGameOver(boolean status){
        gameOver = status;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        if (!drag_enabled && !gameOver){
            if (SwingUtilities.isLeftMouseButton(e)){
                leftClickPressed();
            } else if (SwingUtilities.isRightMouseButton(e)){
                rightClickPressed();
            }
        }
    }

    @Override
    public void mousePressed(MouseEvent e) {
        if (drag_enabled && !gameOver){
            if (SwingUtilities.isLeftMouseButton(e)){
                if (!right_pressed){
                    fist_click = curr_stat;
                    left_pressed = true;
                    mouseEntered(e);
                }
            } else if (SwingUtilities.isRightMouseButton(e)){
                if (!left_pressed){
                    fist_click = curr_stat;
                    right_pressed = true;
                    mouseEntered(e);
                }
            }
        }
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        if (drag_enabled){
            left_pressed = right_pressed = false;
        }
        container.checkBoard();
    }

    @Override
    public void mouseEntered(MouseEvent e) {
        if (container != null){
            container.updateHeaderFocus(ii, jj);
        }
        if (drag_enabled){
            if (left_pressed){
                leftClickPressed();
            } else if (right_pressed){
                rightClickPressed();
            }
        }
    }

    @Override public void mouseExited(MouseEvent e) {}

}
